#include <stdio.h>
#include <plot.h>
#include <math.h>


plPlotter *
open_plotter (FILE *f, double left, double top,
                       double right, double bottom)
{
    plPlotter *p;
    plPlotterParams *p_params;

    /* set a Plotter parameter */
    p_params = pl_newplparams();
    /*
     * pl_setplparam(p_params, "PAGESIZE", "letter,xsize=8in,ysize=1.6in");
     */
    if ((p = pl_newpl_r ("svg", stdin, f, stderr,
                               p_params)) == NULL)
    {
        fprintf (stderr, "Couldn't create Plotter\n");
        return NULL;
    }

    if (pl_openpl_r (p) < 0)      /* open Plotter */
    {
        fprintf (stderr, "Couldn't open Plotter\n");
        return NULL;
    }

    /* set coord system */
    pl_fspace_r (p,  left - 0.5,
                     top - 0.5,
                     right - 0.5,
                     bottom - 0.5);
    pl_erase_r (p);                /* erase graphics display */

    return p;
}


void
close_plotter (plPlotter *plotter)
{
    if (pl_closepl_r (plotter) < 0)     /* close Plotter */
    {
        fprintf (stderr, "Couldn't close Plotter\n");
        return;
    }

    if (pl_deletepl_r (plotter) < 0)    /* delete Plotter */
    {
        fprintf (stderr, "Couldn't delete Plotter\n");
        return;
    }
}


#define S_FUNCTION_LEVEL 2
#define S_FUNCTION_NAME circle_plot
#include "simstruc.h"

#define IN_PORT_0_NAME      x
#define IN_PORT_1_NAME      bg
#define PARAMETER_0_NAME    fileroot
#define PARAMETER_1_NAME    window


#define MDL_CHECK_PARAMETERS
#if defined(MATLAB_MEX_FILE)
/* Function: mdlCheckParameters =============================================
 * Abstract:
 *    Validate our parameters to verify they are okay.
 */
static void mdlCheckParameters(SimStruct *S)
{
    const mxArray *window;
    
    if (ssGetNumSFcnParams(S) != 2)
        ssSetErrorStatus(S,
            "Expecting two parameters, filename and window");
    if (!mxIsChar(ssGetSFcnParam(S, 0)))
        ssSetErrorStatus(S, "Filename parameter must be a string");
    window = ssGetSFcnParam(S, 1);
    if ((mxGetNumberOfElements(window) != 4) &&
        (mxGetNumberOfElements(window) != 0))
        ssSetErrorStatus(S,
            "'window' parameter must have 4 elements or be empty");
}
#endif /* MDL_CHECK_PARAMETERS */



/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *   Setup sizes of the various vectors.
 */
static void mdlInitializeSizes(SimStruct *S)
{
    int i;

    /* set number of parameters (step 1) */
    ssSetNumSFcnParams(S, 2);  /* Number of expected parameters */
    #if defined(MATLAB_MEX_FILE)
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S))
    {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL)
            return;
    }
    else
        return; /* Parameter mismatch will be reported by Simulink */
    #endif /* defined(MATLAB_MEX_FILE) */

    /* Set number of states (calculating derivatives/differences?) (step 2) */
    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    /* Set number of input ports (step 3.1) */
    if (!ssSetNumInputPorts(S, 2)) return;

    /* Specify # input port dimensions (step 3.2) */
    /* Specify input port feedthrough? (step 3.3) */
    for (i = 0; i < 2; i++)
    {
        if(!ssSetInputPortMatrixDimensions(S, i, DYNAMICALLY_SIZED,
                                                 DYNAMICALLY_SIZED)) return;
        ssSetInputPortDirectFeedThrough(S, i, 1); /* It's used in mdlOutputs */
    }

    /* Set number of output ports (step 4) */
    if (!ssSetNumOutputPorts(S, 0)) return;

    /* Set number of sample times (step 5) */
    ssSetNumSampleTimes(S, 1);

    /* Set size of work vectors (extra data storage) (step 6) */
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* Set options (step 7) */
    /* Take care when specifying exception free code - see sfuntmpl_doc.c */
    ssSetOptions(S, 0);
    /*ssSetOptions(S, (SS_OPTION_EXCEPTION_FREE_CODE |
		     SS_OPTION_WORKS_WITH_CODE_REUSE));*/
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    Specifiy  the sample time.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, 0.5);
    /*ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);*/
    ssSetOffsetTime(S, 0, 0.0);
}



#define MDL_SET_INPUT_PORT_DIMENSION_INFO
static void mdlSetInputPortDimensionInfo(SimStruct *S, int_T port,
                                         const DimsInfo_T *dimsInfo)
{
    if (!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
}



/* Function: mdlOutputs =======================================================
 *    DO THINGS HERE
*/
static void mdlOutputs(SimStruct *S, int_T tid)
{
    InputRealPtrsType x = ssGetInputPortRealSignalPtrs (S, 0);
    int_T n_dims = ssGetInputPortNumDimensions (S, 0);
    int_T *x_siz = ssGetInputPortDimensions (S, 0);
    InputRealPtrsType bg = ssGetInputPortRealSignalPtrs (S, 1);
    int_T *bg_siz = ssGetInputPortDimensions (S, 1);
    double pixel_value, center[2], scaling[2], shift[2], pos[2], siz_scale;
    double *window_data, left, right, top, bottom;
    int i, j, k, l, max_i, max_j, max_k, black = 0xbfff, pixel_color[3], ind[2];
    char filename[1024], *fileroot;
    FILE *f;
    plPlotter *p;
    const mxArray *window;

    fileroot = mxArrayToString(ssGetSFcnParam(S, 0));
    sprintf(filename, "%s_%010.06f.svg", fileroot, ssGetT(S));
    f = fopen(filename, "w");

    /* Draw input sequence */
    
    /* Initialize plotting window; check_params has already been called */
    window = ssGetSFcnParam(S, 1);
    if (mxGetNumberOfElements(window) == 4)
    {
        window_data = mxGetPr(window);
        left   = window_data[0];
        top    = window_data[1];
        right  = window_data[2];
        bottom = window_data[3];
    }
    if (!mxGetNumberOfElements(ssGetSFcnParam(S, 1)))
    {
        left   = 0;
        top    = 0;
        right  = bg_siz[1];
        bottom = bg_siz[0];
    }
    max_i = fmin(bg_siz[0], bottom);
    max_j = fmin(bg_siz[1], right);
    p = open_plotter(f, left, top, right, bottom);

    pl_linewidth_r (p, 0);
    pl_filltype_r (p, 0x0001);

    /* Note: this breaks if pixel_value > 1.0 !! */
    for (i = fmax(top, 0); i < max_i; i++)
    {
        for (j = fmax(left, 0); j < max_j; j++)
        {
            pixel_value = *bg[i + bg_siz[0] * j];
            fprintf(stderr, "%d %d: %f\n", i, j, pixel_value);
            for (k = 0; k < 3; k++)
                pixel_color[k] = black + (int) (pixel_value * (0xffff - black));
            pl_color_r (p, pixel_color[0], pixel_color[1], pixel_color[2]);
            pl_fbox_r (p, j - 0.5, i - 0.5, j + 0.5, i + 0.5);
        }
    }

    /* Draw circles */
    max_i = x_siz[0];
    max_j = x_siz[1];

    /* The circles are overlaid on the background, centered and respaced
     * to tile the background
     */
    for (i = 0; i < 2; i++)
    {
        center[i] = (bg_siz[i] - 1.0) / 2.0;
        scaling[i] = ((double) bg_siz[i]) / ((double) x_siz[i]);
        shift[i] = (x_siz[i] - 1.0) / 2.0;
    }
    /* Circles are scaled according to their spacing */
    siz_scale = fmin(scaling[0], scaling[1]) / 2.0;

    pl_colorname_r (p, "black");

    for (j = 0; j < max_j; j++)
    {
        ind[1] = j;
        for (i = 0; i < max_i; i++)
        {
            ind[0] = i;
            for (k = 0; k < 2; k++)
                pos[k] = center[k] + (ind[k] - shift[k]) * scaling[k];
            if (       (pos[0] >= top)
                    && (pos[0] <  bottom)
                    && (pos[1] >= left)
                    && (pos[1] <  right))
                pl_fcircle_r (p, pos[1], pos[0],
                              *x[i + max_i * j] * siz_scale);
        }
    }

    close_plotter (p);
    fclose(f);
}



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
    char command[1024];

    /*sprintf(command, "./fm_visualize %s",
            mxArrayToString(ssGetSFcnParam(S, 0)));
    system(command);*/
}



#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
