#include <stdio.h>
#include <math.h>


#define S_FUNCTION_LEVEL 2
#define S_FUNCTION_NAME my_cont_integrator
#include "simstruc.h"

#define IN_PORT_0_NAME      x
#define PARAMETER_0_NAME    val_max
#define PARAMETER_1_NAME    val_min
static void mdlCheckParameters(SimStruct *S);



/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *   Setup sizes of the various vectors.
 */
static void mdlInitializeSizes(SimStruct *S)
{
    /* set number of parameters (step 1) */
    ssSetNumSFcnParams(S, 2);  /* Number of expected parameters */
    #if defined(MATLAB_MEX_FILE)
    if (ssGetNumSFcnParams(S) == ssGetSFcnParamsCount(S))
    {
        mdlCheckParameters(S);
        if (ssGetErrorStatus(S) != NULL)
            return;
    }
    else
        return; /* Parameter mismatch will be reported by Simulink */
    #endif /* defined(MATLAB_MEX_FILE) */

    /* Set number of states (calculating derivatives/differences?) (step 2) */
    ssSetNumContStates(S, DYNAMICALLY_SIZED);
    ssSetNumDiscStates(S, 0);

    /* Set number of input ports (step 3.1) */
    ssAllowSignalsWithMoreThan2D(S);
    if (!ssSetNumInputPorts(S, 1)) return;
    /* Specify # input port dimensions (step 3.2) */
    if(!ssSetInputPortDimensionInfo(S, 0, DYNAMIC_DIMENSION)) return;
    /* Specify input port feedthrough? (step 3.3) */
    ssSetInputPortDirectFeedThrough(S, 0, 0);

    /* Set number of output ports (step 4) */
    if (!ssSetNumOutputPorts(S, 1)) return;
    if (!ssSetOutputPortDimensionInfo(S, 0, DYNAMIC_DIMENSION)) return;

    /* Set number of sample times (step 5) */
    ssSetNumSampleTimes(S, 1);

    /* Set size of work vectors (extra data storage) (step 6) */
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* Set options (step 7) */
    /* Take care when specifying exception free code - see sfuntmpl_doc.c */
    ssSetOptions(S, 0);
    /*ssSetOptions(S, (SS_OPTION_EXCEPTION_FREE_CODE |
		     SS_OPTION_WORKS_WITH_CODE_REUSE));*/
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    Specifiy  the sample time.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}


#if defined(MATLAB_MEX_FILE)
#define MDL_SET_INPUT_PORT_DIMENSION_INFO
static void mdlSetInputPortDimensionInfo(SimStruct *S,
                                         int_T port,
                                         const DimsInfo_T *dimsInfo)
{
    if(!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
    if(!ssSetOutputPortDimensionInfo(S, port, dimsInfo)) return;
} /* end mdlSetInputPortDimensionInfo */


#define MDL_SET_OUTPUT_PORT_DIMENSION_INFO
static void mdlSetOutputPortDimensionInfo(SimStruct *S,
                                         int_T port,
                                         const DimsInfo_T *dimsInfo)
{
    if(!ssSetOutputPortDimensionInfo(S, port, dimsInfo)) return;
    if(!ssSetInputPortDimensionInfo(S, port, dimsInfo)) return;
} /* end mdlSetOutputPortDimensionInfo */


#define MDL_SET_WORK_WIDTHS
static void mdlSetWorkWidths(SimStruct *S)
{
    ssSetNumContStates(S, ssGetInputPortWidth(S, 0));
}


#define MDL_CHECK_PARAMETERS
static void mdlCheckParameters(SimStruct *S)
{
    const mxArray *param[2];
    int i;

    for (i = 0; i < 2; i++)
    {
        param[i] = ssGetSFcnParam(S, i);
        if (!mxIsDouble(param[i]) || mxIsEmpty(param[i])
                                  || !mxIsScalar(param[i]))
            ssSetErrorStatus(S, "Parameters must be positive scalar doubles");
    }
    if (mxGetScalar(param[0]) < mxGetScalar(param[1]))
        ssSetErrorStatus(S, "Max (param 1) must be more than min (param 2)")
}


#endif /* MATLAB_MEX_FILE */



#define MDL_DERIVATIVES
static void mdlDerivatives(SimStruct *S)
{
    real_T *dx = ssGetdX(S);
    InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    int_T u_siz = ssGetInputPortWidth(S,0);
    int i;

    for (i = 0; i < u_siz; i++)
        dx[i] = *uPtrs[i];
}



/* Function: mdlOutputs =======================================================
 *    DO THINGS HERE
*/
static void mdlOutputs(SimStruct *S, int_T tid)
{
    real_T *x = ssGetContStates(S);
    real_T *y = ssGetOutputPortRealSignal(S, 0);
    int_T siz_y = ssGetOutputPortWidth(S, 0);
    real_T x_max = mxGetScalar(ssGetSFcnParam(S, 0));
    real_T x_min = mxGetScalar(ssGetSFcnParam(S, 1));
    int i;

    for (i = 0; i < siz_y; i++)
    {
        x[i] = fmax (fmin (x[i], x_max), x_min);
        y[i] = x[i];
    }
}



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}



#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
